@@ -1,6 +1,6 @@
 package Reddit::Client;
 
-our $VERSION = '1.04';
+our $VERSION = '1.06';
 $VERSION = eval $VERSION;
 
 use strict;
@@ -19,6 +19,7 @@ require Reddit::Client::Comment;
 require Reddit::Client::Link;
 require Reddit::Client::SubReddit;
 require Reddit::Client::Request;
+require Reddit::Client::Message;
 
 #===============================================================================
 # Constants
@@ -41,6 +42,14 @@ use constant SUBMIT_LINK        => 'link';
 use constant SUBMIT_SELF        => 'self';
 use constant SUBMIT_MESSAGE     => 'message';
 
+use constant MESSAGES_INBOX	     => 'inbox';
+use constant MESSAGES_UNREAD	     => 'unread';
+use constant MESSAGES_SENT	     => 'sent';
+use constant MESSAGES_MESSAGES       => 'messages';
+use constant MESSAGES_COMMENTREPLIES => 'comments';
+use constant MESSAGES_POSTREPLIES    => 'selfreply';
+use constant MESSAGES_MENTIONS	     => 'mentions';
+
 use constant API_ME             => 0;
 use constant API_INFO           => 1;
 use constant API_SEARCH         => 2;
@@ -59,6 +68,9 @@ use constant API_DEL            => 14;
 use constant API_MESSAGE        => 15;
 use constant API_COMMENTS_FRONT	=> 16;
 use constant API_COMMENTS	=> 17;
+use constant API_MESSAGES	=> 18;
+use constant API_MARK_READ	=> 19;
+use constant API_MARKALL	=> 20;
 
 use constant SUBREDDITS_HOME    => '';
 use constant SUBREDDITS_MINE    => 'mine';
@@ -79,7 +91,6 @@ our $UA              = sprintf 'Reddit::Client/%f', $VERSION;
 our @API;
 $API[API_ME            ] = ['GET',  '/api/v1/me'     ];
 $API[API_INFO          ] = ['GET',  '/api/info'      ];
-#$API[API_INFO          ] = ['GET',  '/by_id/%s'      ];
 $API[API_SEARCH        ] = ['GET',  '/reddits/search'];
 $API[API_LOGIN         ] = ['POST', '/api/login/%s'  ];
 $API[API_SUBMIT        ] = ['POST', '/api/submit'    ];
@@ -96,6 +107,9 @@ $API[API_DEL           ] = ['POST', '/api/del'       ];
 $API[API_MESSAGE       ] = ['POST', '/api/compose'   ];
 $API[API_COMMENTS      ] = ['GET',  '/r/%s/comments' ];
 $API[API_COMMENTS_FRONT] = ['GET',  '/comments'      ];
+$API[API_MESSAGES      ] = ['GET',  '/message/%s'    ];
+$API[API_MARK_READ     ] = ['POST', '/api/read_message'];
+$API[API_MARKALL       ] = ['POST', '/api/read_all_messages'];
 #===============================================================================
 # Package routines
 #===============================================================================
@@ -155,11 +169,6 @@ sub new {
     }
     $self->{user_agent} = $param{user_agent};
 
-    #if ($param{session_file}) {
-    #    $self->{session_file} = $param{session_file};
-    #    $self->load_session;
-    #}
-
 	if ($param{username} || $param{password} || $param{client_id} || $param{secret}) {
 		if (!$param{username} || !$param{password} || !$param{client_id} || !$param{secret}) {
 			croak "If any of username, password, client_id, or secret are provided, all are required.";
@@ -225,20 +234,9 @@ sub get_token {
 	if (!$self->{token}) { croak "Unable to get or parse token."; }
 }
 
-# alias for get_token
-sub authorize {
-	my ($self, %rest) = @_;
-	return $self->get_token(%rest);
-}
-
-sub authorized {
-	my $self = shift;
-	return (!$self->{last_token} || $self->{last_token} <= time - 3600) ? 0 : 1;
-}
-
 sub has_token {
 	my $self = shift;
-	return $self->authorized();
+	return (!$self->{last_token} || $self->{last_token} <= time - 3600) ? 0 : 1;
 }
 
 sub json_request {
@@ -253,9 +251,9 @@ sub json_request {
     }
 
     my $response = $self->request($method, $path, $query, $post_data);
-    my $json     = JSON::from_json($response);
 	#use Data::Dump::Color;
-	#dd $json;
+	#dd $response;
+    my $json     = JSON::from_json($response);
 
     if (ref $json eq 'HASH' && $json->{json}) {
         my $result = $json->{json};
@@ -318,103 +316,25 @@ sub api_json_request {
     }
 }
 
+# deprecated, to be removed
 sub is_logged_in {
     return defined $_[0]->{modhash};
 }
 
+# deprecated, to be removed
 sub require_login {
     my $self = shift;
     return;
-    croak 'You must be logged in to perform this action'
-        unless $self->is_logged_in;
 }
 
-sub save_session {
-    my ($self, $file) = @_;
-    $self->require_login;
-    $self->{session_file} || $file || croak 'Expected $file';
-
-    # Prepare session and file path
-    my $session   = { modhash => $self->{modhash}, cookie => $self->{cookie} };
-    my $file_path = File::Path::Expand::expand_filename(
-        defined $file ? $file : $self->{session_file}
-    );
-
-    DEBUG('Save session to %s', $file_path);
-
-    # Write out session
-    open(my $fh, '>', $file_path) or croak $!;
-    print $fh JSON::to_json($session);
-    close $fh;
-
-    # If session file was updated, replace the field
-    $self->{session_file} = $file_path;
-
-    return 1;
-}
-
-sub load_session {
-    my ($self, $file) = @_;
-    $self->{session_file} || $file || croak 'Expected $file';
-    my $file_path = File::Path::Expand::expand_filename(
-        defined $file ? $file : $self->{session_file}
-    );
-
-    DEBUG('Load session from %s', $file_path);
-
-    if (-f $file_path) {
-        open(my $fh, '<', $file_path) or croak $!;
-        my $data = do { local $/; <$fh> };
-        close $fh;
-
-        if ($data) {
-            my $session = JSON::from_json($data);
-            $self->{modhash} = $session->{modhash};
-            $self->{cookie}  = $session->{cookie};
-
-            DEBUG('Session loaded successfully');
-
-            return 1;
-        } else {
-            return 0;
-        }
-    } else {
-        DEBUG('Session file not found');
-        return 0;
-    }
-}
 
 #===============================================================================
 # User and account management
 #===============================================================================
 
-sub authenticate {
-	my ($self, $client_id, $client_secret) = @_;
-}
-
-sub login {
-    my ($self, $usr, $pwd) = @_;
-    !$usr && croak 'Username expected';
-    !$pwd && croak 'Password expected';
-
-    DEBUG('Log in user %s', $usr);
-
-    my $result = $self->api_json_request(
-        api      => API_LOGIN,
-        args     => [$usr],
-        data     => { user => $usr, passwd => $pwd },
-    );
-
-    $self->{modhash} = $result->{data}{modhash};
-    $self->{cookie}  = $result->{data}{cookie};
-
-    return 1;
-}
-
 sub me {
     my $self = shift;
     DEBUG('Request user account info');
-    #$self->require_login;
     my $result = $self->api_json_request(api => API_ME);
     #return Reddit::Client::Account->new($self, $result->{data});
     return Reddit::Client::Account->new($self, $result);
@@ -432,14 +352,56 @@ sub list_subreddits {
     };
 }
 
-sub mod_subreddits     { $_[0]->require_login; return $_[0]->list_subreddits(SUBREDDITS_MOD)     }
-sub my_subreddits      { $_[0]->require_login; return $_[0]->list_subreddits(SUBREDDITS_MINE)    }
-sub contrib_subreddits { $_[0]->require_login; return $_[0]->list_subreddits(SUBREDDITS_CONTRIB) }
-
+sub mod_subreddits     { return $_[0]->list_subreddits(SUBREDDITS_MOD)     }
+sub my_subreddits      { return $_[0]->list_subreddits(SUBREDDITS_MINE)    }
+sub contrib_subreddits { return $_[0]->list_subreddits(SUBREDDITS_CONTRIB) }
 sub home_subreddits    { return $_[0]->list_subreddits(SUBREDDITS_HOME)    }
 sub popular_subreddits { return $_[0]->list_subreddits(SUBREDDITS_POPULAR) }
 sub new_subreddits     { return $_[0]->list_subreddits(SUBREDDITS_NEW)     }
 
+#===============================================================================
+# Inbox and messages
+#===============================================================================
+sub get_inbox {
+	my ($self, %param) = @_;
+    	my $limit     	= $param{limit}		|| DEFAULT_LIMIT;
+	my $mode	= $param{mode}		|| 'inbox';	
+
+	# this before and after business is stupid and needs to be fixed
+	# in 3 separate places
+	my $query = {};
+	$query->{mark}   = $param{mark} ? 'true' : 'false';
+	$query->{sr_detail} = $param{sr_detail} if $param{sr_detail};
+        $query->{before} = $param{before} if $param{before};
+        $query->{after}  = $param{after}  if $param{after};
+	if (exists $param{limit}) { $query->{limit} = $param{limit} || 500; }
+	else 			  { $query->{limit} = DEFAULT_LIMIT;	    }
+	
+	my $result = $self->api_json_request(
+		api	=> API_MESSAGES,
+		args	=> [$mode],
+		data	=> $query,
+	);
+
+	return [
+		map { Reddit::Client::Message->new($self, $_->{data}) } @{$result->{data}{children}}
+	];
+        #map {Reddit::Client::Link->new($self, $_->{data})} @{$result->{data}{children}} 
+}
+
+# TODO
+sub mark_read {
+	my ($self, %param) = @_;
+
+}
+
+sub mark_all_read {
+	my $self = shift;
+	my ($method, $path) = @{$API[API_MARKALL]};
+	my $post_data = {api_type=>'json'};
+	my $result = $self->request($method, $path, {}, $post_data);
+}
+
 #===============================================================================
 # Finding subreddits and listings
 #===============================================================================
@@ -454,7 +416,9 @@ sub info {
 	api => API_INFO, 
 	args => [$id], 
 	data=>$query);
-    return $info->{data}->{children}[0]->{data};
+    my $rtn = $info->{data}->{children}[0]->{data};
+    $rtn->{kind} = $info->{data}->{children}[0]->{kind};
+    return $rtn;
 }
 
 # shortcut to get permalink from info() object
@@ -488,16 +452,12 @@ sub fetch_links {
 
     DEBUG('Fetch %d link(s): %s/%s?before=%s&after=%s', $limit, $subreddit, $view, ($before || '-'), ($after || '-'));
 
-    my $query  = {};
-    if ($before || $after) { 
-        $query->{before} = $before if defined $before;
-        $query->{after}  = $after  if defined $after;
-    }
-	if (exists $param{limit}) {
-		$query->{limit} = $param{limit} || 500;
-	} else {
-		$query->{limit} = DEFAULT_LIMIT;
-	}
+    	my $query  = {};
+        $query->{before} = $param{before} if $param{before};
+        $query->{after}  = $param{after}  if $param{after};
+	if (exists $param{limit}) { $query->{limit} = $param{limit} || 500; }
+	else 			  { $query->{limit} = DEFAULT_LIMIT;	    }
+		
 
     $subreddit = subreddit($subreddit);
 
@@ -510,13 +470,6 @@ sub fetch_links {
         data     => $query,
     );
 
-	# return before and after as a query property instead
-	# this will break EVERYTHING
-
-	# query => {
-        #	before => $result->{data}{before},
-        #	after  => $result->{data}{after},
-	# }
     return [
         map {Reddit::Client::Link->new($self, $_->{data})} @{$result->{data}{children}} 
     ];
@@ -526,20 +479,17 @@ sub get_subreddit_comments {
 	my ($self, %param) = @_;
 	my $subreddit 	= $param{subreddit} 	|| '';
 	my $view 	= $param{view} 		|| VIEW_DEFAULT;
-        my $before    	= $param{before};
-    	my $after     	= $param{after};
 
 	my $query = {};
-        $query->{before} = $before if defined $before;
-        $query->{after}  = $after  if defined $after;
+        #$query->{before} = $before if defined $before;
+        #$query->{after}  = $after  if defined $after;
+        $query->{before} = $param{before} if $param{before};
+        $query->{after}  = $param{after}  if $param{after};
 	# if limit exists but is false (for "no limit"), get as many as possible
 	# this will probably be 100 but ask for a ridiculous amount anyway
 	# if we don't provide a limit, Reddit will give us 25
-	if (exists $param{limit}) {
-		$query->{limit} = $param{limit} || 500;
-	} else {
-		$query->{limit} = DEFAULT_LIMIT;
-	}
+	if (exists $param{limit}) { $query->{limit} = $param{limit} || 500; }
+	else 			  { $query->{limit} = DEFAULT_LIMIT;	    }
 
 	$subreddit = subreddit($subreddit); # remove slashes and leading r/
     	my $args = [$view];
@@ -550,12 +500,7 @@ sub get_subreddit_comments {
         	args     => $args,
         	data     => $query,
     	);
-	#use Data::Dump::Color;
-	#dd $result;
 
-        	#before => $result->{data}{before},
-        	#after  => $result->{data}{after},
-		#items => []
     	return [
         	 map {Reddit::Client::Comment->new($self, $_->{data})} @{$result->{data}{children}} 
     	];
@@ -586,7 +531,7 @@ sub submit_link {
     my $subreddit = $param{subreddit} || '';
     my $title     = $param{title}     || croak 'Expected "title"';
     my $url       = $param{url}       || croak 'Expected "url"';
-    $self->require_login;
+	my $replies = exists $param{inbox_replies} ? $param{inbox_replies} : 1;
 
     DEBUG('Submit link to %s: %s', $subreddit, $title, $url);
 
@@ -597,6 +542,7 @@ sub submit_link {
         url      => $url,
         sr       => $subreddit,
         kind     => SUBMIT_LINK,
+	sendreplies=>$replies,
     });
 
     return $result->{data}{name};
@@ -607,7 +553,7 @@ sub submit_text {
     my $subreddit = $param{subreddit} || '';
     my $title     = $param{title}     || croak 'Expected "title"';
     my $text      = $param{text}      || croak 'Expected "text"';
-    $self->require_login;
+	my $replies = exists $param{inbox_replies} ? $param{inbox_replies} : 1;
 
     DEBUG('Submit text to %s: %s', $subreddit, $title);
 
@@ -618,6 +564,7 @@ sub submit_text {
         text     => $text,
         sr       => $subreddit,
         kind     => SUBMIT_SELF,
+	sendreplies=>$replies,
     });
 
     return $result->{data}{name};
@@ -689,7 +636,6 @@ sub vote {
     defined $direction || croak 'Expected $direction';
     DEBUG('Vote %d for %s', $direction, $name);
     croak 'Invalid vote direction' unless "$direction" =~ /^(-1|0|1)$/;
-    $self->require_login;
     $self->api_json_request(api => API_VOTE, data => { dir => $direction, id  => $name });
 }
 
@@ -701,7 +647,6 @@ sub save {
     my $self = shift;
     my $name = shift || croak 'Expected $name';
     DEBUG('Save %s', $name);
-    $self->require_login;
     $self->api_json_request(api => API_SAVE, data => { id => $name });
 }
 
@@ -709,7 +654,6 @@ sub unsave {
     my $self = shift;
     my $name = shift || croak 'Expected $name';
     DEBUG('Unsave %s', $name);
-    $self->require_login;
     $self->api_json_request(api => API_UNSAVE, data => { id => $name });
 }
 
@@ -717,7 +661,6 @@ sub hide {
     my $self = shift;
     my $name = shift || croak 'Expected $name';
     DEBUG('Hide %s', $name);
-    $self->require_login;
     $self->api_json_request(api => API_HIDE, data => { id => $name });
 }
 
@@ -725,7 +668,6 @@ sub unhide {
     my $self = shift;
     my $name = shift || croak 'Expected $name';
     DEBUG('Unhide %s', $name);
-    $self->require_login;
     $self->api_json_request(api => API_UNHIDE, data => { id => $name });
 }
 
@@ -743,10 +685,10 @@ Reddit::Client - A Perl wrapper for the Reddit API.
 
     use Reddit::Client;
 
-    my $client_id  = "DFhtrhBgfhhRTd";
-    my $secret     = "KrDNsbeffdbILOdgbgSvSBsbfFs";
-    my $username   = "reddit_username";
-    my $password   = "reddit_password";
+    my $client_id  	= "DFhtrhBgfhhRTd";
+    my $secret     	= "KrDNsbeffdbILOdgbgSvSBsbfFs";
+    my $username   	= "reddit_username";
+    my $password   	= "reddit_password";
 
 
     # Create a Reddit::Client object and authorize in one step
@@ -760,8 +702,8 @@ Reddit::Client - A Perl wrapper for the Reddit API.
 	
     # Or create object then authorize.
     # Useful if you need to switch between accounts, for example if you were to check the inboxes of several accounts.
-    my $reddit       = Reddit::Client->new(
-        user_agent   => 'MyApp/1.0',
+    my $reddit = Reddit::Client->new(
+        user_agent   	=> 'MyApp/1.0 by /u/earth-tone',
     );
 
     $reddit->get_token(
@@ -877,6 +819,10 @@ Return true if a valid Oauth token exists.
 
 Returns a Reddit::Client::Account object.
 
+=item mark_all_read
+
+Mark your inbox read. May not happen instantly.
+
 
 =item info($fullname)
 
@@ -894,32 +840,32 @@ is a C<SUBREDDITS_*> constant. A number of alias functions are available:
 
 =item my_subreddits
 
-Syntactic sugar for C<list_subreddits(SUBREDDITS_MINE)>. 
+Alias for C<list_subreddits(SUBREDDITS_MINE)>. 
 
 
 =item home_subreddits
 
-Syntactic sugar for C<list_subreddits(SUBREDDITS_HOME)>. 
+Alias for C<list_subreddits(SUBREDDITS_HOME)>. 
 
 
 =item mod_subreddits
 
-Syntactic sugar for C<list_subreddits(SUBREDDITS_MOD)>. 
+Alias for C<list_subreddits(SUBREDDITS_MOD)>. 
 
 
 =item contrib_subreddits
 
-Syntactic sugar for C<list_subreddits(SUBREDDITS_CONTRIB)>.
+Alias for C<list_subreddits(SUBREDDITS_CONTRIB)>.
 
 
 =item popular_subreddits
 
-Syntactic sugar for C<list_subreddits(SUBREDDITS_POPULAR)>.
+Alias for C<list_subreddits(SUBREDDITS_POPULAR)>.
 
 
 =item new_subreddits
 
-Syntactic sugar for C<list_subreddits(SUBREDDITS_NEW)>.
+Alias for C<list_subreddits(SUBREDDITS_NEW)>.
 
 =back
 
@@ -931,7 +877,7 @@ Returns a list of SubReddit objects matching C<$query>.
 
 =item fetch_links([subreddit => ...], [view => ...], [limit => ...], [before => ...], [after => ...])
 
-Returns a list of Link objects. To get at the posts, iterate the C<items> array.
+Returns a list of Link objects. 
 
 All arguments are optional. If C<subreddit> is specified,
 the list of links is returned from the desired subreddit or multi (i.e. "pics+new"). Otherwise, the
@@ -947,14 +893,14 @@ as a hash with three keys, I<before>, I<after>, and I<items>.
 Deletes a post or comment. The object's fullname is required.
 
 
-=item submit_link(subreddit => ..., title => ..., url => ...)
+=item submit_link(subreddit => ..., title => ..., url => ... [inbox_replies => ...])
 
-Submits a link to a reddit. Returns the id of the new link.
+Submits a link to a reddit. Returns the fullname of the new link. If inbox_replies is set and is false, disable inbox replies for that post.
 
 
-=item submit_text(subreddit => ..., title => ..., text => ...)
+=item submit_text(subreddit => ..., title => ..., text => ... [inbox_replies => ...)
 
-Submits a self-post to a reddit. Returns the id of the new post.
+Submits a self-post to a reddit. Returns the fullname of the new post. If inbox_replies is set and is false, disable inbox replies for that post.
  
 
 =item get_subreddit_comments([subreddit => ...][before => ...][after => ...][limit => ...])
