@@ -7,7 +7,7 @@ use Carp;
 require Reddit::Client::VotableThing;
 
 use base   qw/Reddit::Client::VotableThing/;
-use fields qw/link_flair_text media url link_flair_css_class num_reports created_utc
+use fields qw/link_flair_text media url link_url link_flair_css_class num_reports created_utc
 			  banned_by subreddit title author_flair_text is_self author media_embed
 			  permalink author_flair_css_class selftext domain num_comments clicked
 			  saved thumbnail subreddit_id approved_by selftext_html created hidden
@@ -72,6 +72,8 @@ with no replies return an empty array for C<replies>.
 
 =head1 AUTHOR
 
+<mailto:earth-tone@ubwg.net>
+
 Jeff Ober L<mailto:jeffober@gmail.com>
 
 =head1 LICENSE
